require_relative './base'

module Zpl
  # This transformer will remove all commands given by name.
  #
  # Example: This will remove the command named `MM`.
  # ```rb
  # code = '^LT0^LS0^MMT^XA^XZ'
  # remover = Zpl::Transformer::CmdRemover.new 'MM'
  # remover.apply(code) # => "^LT0^LS0^XA^XZ"
  # ```
  class Transformer::CmdRemover < Transformer::Base

    def initialize(*cmds_to_delete)
      @cmds_to_delete = Set.new cmds_to_delete.flatten
    end

    def map_cmd cmd
      cmd unless @cmds_to_delete.include?(cmd.name)
    end

  end
end
